{-# LANGUAGE OverloadedStrings #-}

module HAL where

import Data.Default (def)
import Data.Text

import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)

import Servant.Client (BaseUrl(..), Scheme(..), ClientM, ClientError, runClientM, mkClientEnv)

import HAL.Client
import HAL.Doc.Corpus


getMetadataWith :: Text -> Maybe Int -> IO (Either ClientError (Response Corpus))
getMetadataWith q l = do
  manager' <- newManager tlsManagerSettings
  runHalAPIClient $ search (Just requestedFields) [q] Nothing l Nothing

requestedFields :: Text
requestedFields = "docid,title_s,abstract_s,submittedDate_s,source_s,authFullName_s,authOrganism_s"

runHalAPIClient :: ClientM (Response Corpus) -> IO (Either ClientError (Response Corpus))
runHalAPIClient cmd = do
  manager' <- newManager tlsManagerSettings
  runClientM cmd (mkClientEnv manager' $ BaseUrl Https "api.archives-ouvertes.fr" 443 "")

runStructureRequest :: Maybe Text -> IO (Either ClientError (Response Corpus))
runStructureRequest rq =
  runHalAPIClient $ structure (Just requestedFields) rq (Just 10000)

runSearchRequest :: [Text] -> IO (Either ClientError (Response Corpus))
runSearchRequest rq =
  runHalAPIClient $ search (Just requestedFields) rq Nothing Nothing Nothing
