{-# LANGUAGE DataKinds #-}
{-# LANGUAGE TypeOperators #-}
{-# LANGUAGE OverloadedStrings #-}

module Tree where

import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Servant.Client hiding (Response)
import Control.Lens.Getter ((^.))
import Data.Map ((!?), (!), insert, empty, Map, fromList, toList)
import qualified Data.Map as M
import Data.Map.Internal (merge, preserveMissing, zipWithMatched)
import Data.List (groupBy, isInfixOf)
import Data.List.Split (chunksOf)
import Data.Either (rights)
import Data.Maybe (fromMaybe)
import Data.Scientific (Scientific, floatingOrInteger)
import qualified Data.Vector as V
import Text.Printf

import HAL.Client
import HAL.Doc
{-
formatParentIdRequest :: [Doc] -> Maybe Text
formatParentIdRequest [] = Nothing
formatParentIdRequest (x:[]) = Just . pack . show $ _docid x
formatParentIdRequest (x:xs) =
  (Just . pack . show $ _docid x)
  <> (Just " || ")
  <> formatParentIdRequest xs


ds2Child :: [Doc] -> IO [Doc]
ds2Child ds =  do
  rs <- sequence $ runStructureRequest <$> formatedRequests ds
  return . concat $ (^. docs) <$> rights rs
  where formatedRequest docs' = Just "parentDocid_i:(" <> formatParentIdRequest docs' <> Just ")"
        formatedRequests docs'' = formatedRequest <$> chunksOf 100 docs''

fetchChildren :: [Doc] -> IO [Doc]
fetchChildren [] = pure []
fetchChildren ds = (ds <>) <$> (fetchChildren =<< ds2Child ds)

isChildOf :: Doc -> Doc -> Bool
isChildOf (Doc i l) (Doc i' l') = not . null $ V.filter (\id -> id == (scientific2text docId)) ((\(String a) -> a) <$> parentDocIds)
  where (Number docId ) = d ! "docid"
        (Array parentDocIds) = d' ! "parentDocid_i"

data DocTree = DocTree Doc Int [DocTree]
  deriving (Show, Generic)
instance ToJSON DocTree

buildTree :: Int -> [Doc] -> Doc -> DocTree
buildTree depth docs id = DocTree id depth (buildTree (depth + 1) docs <$> children)
  where children = filter (isChildOf id) docs

formatTree :: DocTree -> String
formatTree tree@(DocTree doc depth children) =
  printf "%s%s\n%s" (addSpace) (show doc) (concat $ formatTree <$> children)
  where addSpace
          | depth > 0 = "├" <> (replicate (depth * depth + depth) '─')
          | otherwise = "🌲"

findDeepest :: Map Scientific Int -> DocTree -> Map Scientific Int
findDeepest m tree@(DocTree doc depth children) =
  mergeMap map1 maps
  where map1 = insert docid' depth m
        maps = foldl mergeMap empty $ findDeepest m <$> children
        mergeMap m1 m2 = merge preserveMissing preserveMissing (zipWithMatched whenMatch) m1 m2
        whenMatch k v v' = if v > v' then v else v'
        (Number docid') = docid doc

isDeep :: Map Scientific Int -> DocTree -> Bool
isDeep m (DocTree i depth _) = depth >= (fromMaybe 0 $ M.lookup id m)
  where (Number id) = docid i

removeDuplicate :: Map Scientific Int -> DocTree -> DocTree
removeDuplicate deepMap tree@(DocTree doc depth children) = DocTree doc depth (removeDuplicate deepMap <$> filter (isDeep deepMap) children)

noDuplicateTree :: DocTree -> DocTree
noDuplicateTree tree = removeDuplicate (findDeepest M.empty tree) tree

-}
